<?php
/* Copyright (C) 2021      Open-DSI             <support@open-dsi.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * \file        htdocs/advancedcrm/class/advancedcrmmailing.class.php
 * \ingroup     advancedcrm
 * \brief       This file is a CRUD class file for AdvancedCrmMailing (Create/Read/Update/Delete)
 */

/**
 * Class for TaskToAgenda
 */
class TaskToAgenda
{
	/**
	 * @var DoliDb		Database handler (result of a new DoliDB)
	 */
	public $db;
	/**
	 * @var string 		Error string
	 * @see             $errors
	 */
	public $error;
	/**
	 * @var string[]	Array of error strings
	 */
	public $errors = array();
	/**
	 * @var string[]	Array of success strings
	 */
	public $successes = array();

	/**
	 * Type link event
	 */
	const EVENT_TYPE_ID = 163001;

	/**
	 * Type link event
	 */
	const EVENT_ELEMENT_TYPE = 'task';

	/**
	 * Constructor
	 *
	 * @param DoliDb $db Database handler
	 */
	public function __construct(DoliDB $db)
	{
		$this->db = $db;
	}

	/**
	 *  Create event from task object into database
	 *
	 * @param	User	$user			User that creates
	 * @param	int		$task_id		Task ID
	 * @param	bool	$notrigger		false=launch triggers after, true=disable triggers
	 * @return	int						<0 if KO, Id of created object if OK
	 */
	public function createEventFromTask(User $user, $task_id, $notrigger = false)
	{
		dol_syslog(__METHOD__ . " user_id={$user->id}, task_id={$task_id}, no_trigger={$notrigger}", LOG_DEBUG);
		global $conf, $langs;
		$langs->load('tasktoagenda@tasktoagenda');

		// Clean parameters
		$task_id = $task_id > 0 ? $task_id : 0;

		// Check parameters
		if (empty($task_id)) {
			$langs->load('errors');
			$this->error = $langs->trans('ErrorBadParameters') . " (Task ID)";
			return -1;
		}

		require_once DOL_DOCUMENT_ROOT . '/projet/class/task.class.php';
		$task = new Task($this->db);
		$result = $task->fetch($task_id);
		if ($result < 0) {
			$this->error = $task->error;
			$this->errors = $task->errors;
			return -1;
		} elseif ($result == 0) {
			$langs->load('errors');
			$this->error = $langs->trans('ErrorRecordNotFound') . " (Task ID: {$task_id})";
			return -1;
		}

		require_once DOL_DOCUMENT_ROOT . '/projet/class/project.class.php';
		$project = new Project($this->db);
		$result = $project->fetch($task->fk_project);
		if ($result < 0) {
			$this->error = $project->error;
			$this->errors = $project->errors;
			return -1;
		} elseif ($result == 0) {
			$langs->load('errors');
			$this->error = $langs->trans('ErrorRecordNotFound') . " (Project ID: {$task->fk_project})";
			return -1;
		}
		$project->fetch_thirdparty();

		require_once DOL_DOCUMENT_ROOT . '/comm/action/class/actioncomm.class.php';
		$event = new ActionComm($this->db);

		$event->context['tta_processing'] = true;
		$event->type_id = self::EVENT_TYPE_ID;
		$event->type_code = 'AC_TTA_TASK';
		$event->priority = 0;
		$event->fulldayevent = 0;
		$event->location = '';
		$event->label = $task->label;
		$event->userownerid = $user->id;
		$event->fk_element = $task->id;
		$event->elementtype = self::EVENT_ELEMENT_TYPE;
		$event->fk_project = $task->fk_project;
		$event->datep = $task->date_start;
		$event->datef = $task->date_end;
		$event->percentage = isset($task->progress) ? $task->progress : 0;
		$event->note_private = $task->description;
		$event->socid = $project->socid;
		$event->societe = $project->thirdparty;    // For backward compatibility

		// Get contacts
		$result = $task->liste_contact(-1, 'external');
		if (!is_array($result)) {
			$this->error = $task->error;
			$this->errors = $task->errors;
			return -1;
		}
		if (!empty($conf->global->TASKTOAGENDA_SYNC_PROJECT_CONTACTS)) {
			$result2 = $project->liste_contact(-1, 'external');
			if (!is_array($result2)) {
				$this->error = $task->error;
				$this->errors = $task->errors;
				return -1;
			}
			$result = array_merge(array_values($result), array_values($result2));
		}
		$contacts_list = array();
		$assigned_contacts = array();
		foreach ($result as $infos) {
			$assigned_contacts[$infos['id']] = array('id' => $infos['id'], 'transparency' => 1);
			$contacts_list[$infos['id']] = $infos;
		}
		$event->socpeopleassigned = $assigned_contacts;

		// Get users
		$result = $task->liste_contact(-1, 'internal');
		if (!is_array($result)) {
			$this->error = $task->error;
			$this->errors = $task->errors;
			return -1;
		}
		if (!empty($conf->global->TASKTOAGENDA_SYNC_PROJECT_USERS)) {
			$result2 = $project->liste_contact(-1, 'internal');
			if (!is_array($result2)) {
				$this->error = $task->error;
				$this->errors = $task->errors;
				return -1;
			}
			$result = array_merge(array_values($result), array_values($result2));
		}
		$users_list = array();
		$assigned_users = array();
		foreach ($result as $infos) {
			$assigned_users[$infos['id']] = array('id' => $infos['id'], 'transparency' => 1);
			$users_list[$infos['id']] = $infos;
		}
		$assigned_users[$user->id] = array('id' => $user->id, 'transparency' => 1);
		$users_list[$user->id] = array(
			'id' => $user->id,
			'firstname' => $user->firstname,
			'lastname' => $user->lastname,
			'email' => $user->email,
		);
		$event->userassigned = $assigned_users;

		$error = 0;
		$this->db->begin();

		// Create event
		$event_id = $event->create($user);
		if ($event_id < 0) {
			$this->error = $event->error;
			$this->errors = $event->errors;
			$error++;
		}

		// Notify by mail
		if (!$error && !empty($users_list) && !empty($conf->global->TASKTOAGENDA_NOTIFY_BY_EMAIL_WHEN_CREATE)) {
			foreach ($users_list as $infos) {
				$result = $this->sendNotification($user, $project, $task, $infos);
				if ($result < 0) {
					$error++;
					break;
				}
			}
		}

		if ($error) {
			$this->db->rollback();
			return -1;
		} else {
			$this->db->commit();
		}

		return $event_id;
	}

	/**
	 *  Update event from task object into database
	 *
	 * @param	User	$user				User that creates
	 * @param	Task	$object				Task handler
	 * @param	bool	$is_time_updated	Is time create/update trigger called
	 * @param	bool	$notrigger			false=launch triggers after, true=disable triggers
	 * @return	int							<0 if KO, Id of created object if OK
	 */
	public function updateEventFromTask(User $user, &$object, $is_time_updated, $notrigger = false)
	{
		dol_syslog(__METHOD__ . " user_id={$user->id}, task_id={$object->id}, is_time_updated={$is_time_updated}, no_trigger={$notrigger}", LOG_DEBUG);
		global $conf, $langs;
		$langs->load('tasktoagenda@tasktoagenda');

		// Clean parameters
		$task_id = $object->id > 0 ? $object->id : 0;

		// Check parameters
		if (empty($task_id)) {
			$langs->load('errors');
			$this->error = $langs->trans('ErrorBadParameters') . " (Task ID)";
			return -1;
		}

		require_once DOL_DOCUMENT_ROOT . '/projet/class/task.class.php';
		$task = new Task($this->db);
		$result = $task->fetch($task_id);
		if ($result < 0) {
			$this->error = $task->error;
			$this->errors = $task->errors;
			return -1;
		} elseif ($result == 0) {
			$langs->load('errors');
			$this->error = $langs->trans('ErrorRecordNotFound') . " (Task ID: {$task_id})";
			return -1;
		}

		require_once DOL_DOCUMENT_ROOT . '/projet/class/project.class.php';
		$project = new Project($this->db);
		$result = $project->fetch($task->fk_project);
		if ($result < 0) {
			$this->error = $project->error;
			$this->errors = $project->errors;
			return -1;
		} elseif ($result == 0) {
			$langs->load('errors');
			$this->error = $langs->trans('ErrorRecordNotFound') . " (Project ID: {$task->fk_project})";
			return -1;
		}
		$project->fetch_thirdparty();

		$event = $this->getEvent($task_id);
		if (!is_object($event)) {
			return -1;
		} elseif (empty($event->id)) {
			// Don't have a event to update
			return 1;
		}

		$notify = false;
		if (!empty($conf->global->TASKTOAGENDA_NOTIFY_BY_EMAIL_WHEN_DATES_UPDATED) &&
			($event->datep != $task->date_start || $event->datef != $task->date_end))
			$notify = true;
		if (!empty($conf->global->TASKTOAGENDA_NOTIFY_BY_EMAIL_WHEN_PROGRESS_UPDATED) && (isset($event->percentage) ? $event->percentage : 0) != (isset($task->progress) ? $task->progress : 0))
			$notify = true;
		if (!empty($conf->global->TASKTOAGENDA_NOTIFY_BY_EMAIL_WHEN_DESCRIPTION_UPDATED) && trim($event->note_private) != trim($task->description))
			$notify = true;

		$event->context['tta_processing'] = true;
		$event->label = $task->label;
		$event->fk_project = $task->fk_project;
		$event->datep = $task->date_start;
		$event->datef = $task->date_end;
		$event->percentage = $is_time_updated && isset($object->progress) ? $object->progress : (isset($task->progress) ? $task->progress : 0);
		$event->note_private = $task->description;
		$event->socid = $project->socid;
		$event->societe = $project->thirdparty;    // For backward compatibility

		// Get contacts
		$old_contacts_list = array();
		if (is_array($event->socpeopleassigned)) {
			require_once DOL_DOCUMENT_ROOT . '/contact/class/contact.class.php';
			$contact_static = new Contact($this->db);
			foreach ($event->socpeopleassigned as $infos) {
				$result = $contact_static->fetch($infos['id']);
				if ($result > 0) {
					$old_contacts_list[$infos['id']] = array(
						'id' => $infos['id'],
						'firstname' => $contact_static->firstname,
						'lastname' => $contact_static->lastname,
						'email' => $contact_static->email,
					);
				} elseif ($result < 0) {
					$this->error = $contact_static->error;
					$this->errors = $contact_static->errors;
					return -1;
				}
			}
		}
		$result = $task->liste_contact(-1, 'external');
		if (!is_array($result)) {
			$this->error = $task->error;
			$this->errors = $task->errors;
			return -1;
		}
		if (!empty($conf->global->TASKTOAGENDA_SYNC_PROJECT_CONTACTS)) {
			$result2 = $project->liste_contact(-1, 'external');
			if (!is_array($result2)) {
				$this->error = $task->error;
				$this->errors = $task->errors;
				return -1;
			}
			$result = array_merge(array_values($result), array_values($result2));
		}
		$contacts_list = array();
		$assigned_contacts = array();
		foreach ($result as $infos) {
			$assigned_contacts[$infos['id']] = array('id' => $infos['id'], 'transparency' => 1);
			$contacts_list[$infos['id']] = $infos;
		}
		$event->socpeopleassigned = $assigned_contacts;

		// Get users
		$old_users_list = array();
		if (is_array($event->userassigned)) {
			require_once DOL_DOCUMENT_ROOT . '/user/class/user.class.php';
			$user_static = new User($this->db);
			foreach ($event->userassigned as $infos) {
				$result = $user_static->fetch($infos['id']);
				if ($result > 0) {
					$old_users_list[$infos['id']] = array(
						'id' => $infos['id'],
						'firstname' => $user_static->firstname,
						'lastname' => $user_static->lastname,
						'email' => $user_static->email,
					);
				} elseif ($result < 0) {
					$this->error = $user_static->error;
					$this->errors = $user_static->errors;
					return -1;
				}
			}
		}
		$result = $task->liste_contact(-1, 'internal');
		if (!is_array($result)) {
			$this->error = $task->error;
			$this->errors = $task->errors;
			return -1;
		}
		if (!empty($conf->global->TASKTOAGENDA_SYNC_PROJECT_USERS)) {
			$result2 = $project->liste_contact(-1, 'internal');
			if (!is_array($result2)) {
				$this->error = $task->error;
				$this->errors = $task->errors;
				return -1;
			}
			$result = array_merge(array_values($result), array_values($result2));
		}
		$users_list = array();
		$assigned_users = array();
		foreach ($result as $infos) {
			$assigned_users[$infos['id']] = array('id' => $infos['id'], 'transparency' => 1);
			$users_list[$infos['id']] = $infos;
		}
		$assigned_users[$user->id] = array('id' => $user->id, 'transparency' => 1);
		$event->userassigned = $assigned_users;

		$error = 0;
		$this->db->begin();

		// Create event
		$result = $event->update($user);
		if ($result < 0) {
			$this->error = $event->error;
			$this->errors = $event->errors;
			$error++;
		}

		// Notify by mail
		$update_users_list = array_intersect_key($old_users_list, $users_list);
		$assign_users_list = array_diff_key($users_list, $old_users_list);
		$unassign_users_list = array_diff_key($old_users_list, $users_list);

		if (!$error && $notify && !empty($update_users_list)) {
			foreach ($update_users_list as $infos) {
				$result = $this->sendNotification($user, $project, $task, $infos, 1);
				if ($result < 0) {
					$error++;
					break;
				}
			}
		}
		if (!$error && !empty($assign_users_list)) {
			foreach ($assign_users_list as $infos) {
				$result = $this->sendNotification($user, $project, $task, $infos, 2);
				if ($result < 0) {
					$error++;
					break;
				}
			}
		}
		if (!$error && !empty($unassign_users_list)) {
			foreach ($unassign_users_list as $infos) {
				$result = $this->sendNotification($user, $project, $task, $infos, 3);
				if ($result < 0) {
					$error++;
					break;
				}
			}
		}

		if ($error) {
			$this->db->rollback();
			return -1;
		} else {
			$this->db->commit();
		}

		return 1;
	}

	/**
	 *  Update task from event object into database
	 *
	 * @param	User			$user			User that creates
	 * @param	ActionComm		&$event			Event handler
	 * @param	bool			$notrigger		false=launch triggers after, true=disable triggers
	 * @return	int								<0 if KO, Id of created object if OK
	 */
	public function updateTaskFromEvent(User $user, &$event, $notrigger = false)
	{
		dol_syslog(__METHOD__ . " user_id={$user->id}, event_id={$event->id}, no_trigger={$notrigger}", LOG_DEBUG);
		global $conf, $langs;
		$langs->load('tasktoagenda@tasktoagenda');

		// Check parameters
		if (!is_object($event) || !($event->id > 0)) {
			$langs->load('errors');
			$this->error = $langs->trans('ErrorBadParameters') . " (Event)";
			return -1;
		}

		$notify = false;
		$error = 0;
		$no_old_copy = !is_object($event->oldcopy);

		// Get old event
		if ($no_old_copy) {
			// Cancel last event updated
			if (isset($this->db->db->transaction_opened)) {
				$transaction_opened = $this->db->db->transaction_opened;
				$this->db->db->transaction_opened = 1;
			} else {
				$transaction_opened = $this->db->transaction_opened;
				$this->db->transaction_opened = 1;
			}
			$this->db->rollback();
			$this->db->begin();
			if (isset($this->db->db->transaction_opened)) {
				$this->db->db->transaction_opened = $transaction_opened;
			} else {
				$this->db->transaction_opened = $transaction_opened;
			}

			// Update event
			$old_event = new ActionComm($this->db);
			$old_event->fetch($event->id);
		} else {
			$old_event = $event->oldcopy;
		}

		// If the old event is not a type 'link task/event' and the updated event is a type 'link task/event'
		// If the old event is a type 'link task/event' and the updated event is not a type 'link task/event'
		if (($old_event->type_id != self::EVENT_TYPE_ID && $event->type_id == self::EVENT_TYPE_ID) ||
			($old_event->type_id == self::EVENT_TYPE_ID && $event->type_id != self::EVENT_TYPE_ID)
		) {
			$this->error = $langs->trans('TaskToAgendaErrorCannotModifyTypeOfEventLinkType');
			return -1;
		}

		// If the old event is not a type 'link task/event' and the updated event is not a type 'link task/event' then keep updated event
		// If the old event is a type 'link task/event' and the updated event is a type 'link task/event' then keep updated event
		if ($no_old_copy) {
			$event->context['tta_processing'] = true;
			$event->update($user);
		}

		// If the old event is not a type 'link task/event' and the updated event is not a type 'link task/event' then do nothing
		if ($old_event->type_id != self::EVENT_TYPE_ID && $event->type_id != self::EVENT_TYPE_ID)
			return 0;

		require_once DOL_DOCUMENT_ROOT . '/projet/class/task.class.php';
		$task = new Task($this->db);
		$result = $task->fetch($event->elementid);
		if ($result < 0) {
			$this->error = $task->error;
			$this->errors = $task->errors;
			$error++;
		} elseif ($result == 0) {
			$langs->load('errors');
			$this->error = $langs->trans('ErrorRecordNotFound') . " (Task ID: {$event->elementid})";
			$error++;
		}

		if (!$error) {
			require_once DOL_DOCUMENT_ROOT . '/projet/class/project.class.php';
			$project = new Project($this->db);
			$result = $project->fetch($task->fk_project);
			if ($result < 0) {
				$this->error = $project->error;
				$this->errors = $project->errors;
				$error++;
			} elseif ($result == 0) {
				$langs->load('errors');
				$this->error = $langs->trans('ErrorRecordNotFound') . " (Project ID: {$task->fk_project})";
				$error++;
			}
			$project->fetch_thirdparty();
		}

		// Update task
		if (!$error) {
			if (!empty($conf->global->TASKTOAGENDA_NOTIFY_BY_EMAIL_WHEN_DATES_UPDATED) && ($task->date_start != $event->datep || $task->date_end != $event->datef))
				$notify = true;
			if (!empty($conf->global->TASKTOAGENDA_NOTIFY_BY_EMAIL_WHEN_PROGRESS_UPDATED) && (isset($task->progress) ? $task->progress : 0) != (isset($event->percentage) ? $event->percentage : 0))
				$notify = true;
			if (!empty($conf->global->TASKTOAGENDA_NOTIFY_BY_EMAIL_WHEN_DESCRIPTION_UPDATED) && trim($task->description) != trim($event->note_private))
				$notify = true;

			$task->context['tta_processing'] = true;
			$task->label = $event->label;
			$task->date_start = $event->datep;
			$task->date_end = $event->datef;
			$task->progress = $event->percentage;
			$task->description = $event->note_private;
			$result = $task->update($user);
			if ($result < 0) {
				$this->error = $task->error;
				$this->errors = $task->errors;
				$error++;
			}
		}

		if (!$error && $notify) {
			// Get users
			$users_list = $task->liste_contact(-1, 'internal');
			if (!is_array($users_list)) {
				$this->error = $task->error;
				$this->errors = $task->errors;
				$error++;
			}

			// Notify by mail
			if (!$error && !empty($users_list)) {
				foreach ($users_list as $infos) {
					$result = $this->sendNotification($user, $project, $task, $infos, 1);
					if ($result < 0) {
						$error++;
						break;
					}
				}
			}
		}

		if ($error) {
			return -1;
		}

		return 1;
	}

	/**
	 *  Delete event from task object into database
	 *
	 * @param	User	$user			User that creates
	 * @param	Task	&$task			Task handler
	 * @param	bool	$notrigger		false=launch triggers after, true=disable triggers
	 * @return	int						<0 if KO, Id of created object if OK
	 */
	public function deleteEventFromTask(User $user, $task, $notrigger = false)
	{
		dol_syslog(__METHOD__ . " user_id={$user->id}, task_id={$task->id}, no_trigger={$notrigger}", LOG_DEBUG);
		global $conf, $langs;
		$langs->load('tasktoagenda@tasktoagenda');

		// Check parameters
		if (!is_object($task) || !($task->id > 0)) {
			$langs->load('errors');
			$this->error = $langs->trans('ErrorBadParameters') . " (Task)";
			return -1;
		}

		require_once DOL_DOCUMENT_ROOT . '/projet/class/project.class.php';
		$project = new Project($this->db);
		$result = $project->fetch($task->fk_project);
		if ($result < 0) {
			$this->error = $project->error;
			$this->errors = $project->errors;
			return -1;
		} elseif ($result == 0) {
			$langs->load('errors');
			$this->error = $langs->trans('ErrorRecordNotFound') . " (Project ID: {$task->fk_project})";
			return -1;
		}
		$project->fetch_thirdparty();

		$event = $this->getEvent($task->id);
		if (!is_object($event)) {
			return -1;
		} elseif (empty($event->id)) {
			// Don't have a event to delete
			return 1;
		}

		$event->context['tta_processing'] = true;

		$error = 0;
		$this->db->begin();

		// Create event
		$result = $event->delete($user);
		if ($result < 0) {
			$this->error = $event->error;
			$this->errors = $event->errors;
			$error++;
		}

		// Notify by mail
		if (!$error && !empty($event->userassigned) && !empty($conf->global->TASKTOAGENDA_NOTIFY_BY_EMAIL_WHEN_DELETE)) {
			require_once DOL_DOCUMENT_ROOT . '/user/class/user.class.php';
			$user_static = new User($this->db);
			foreach ($event->userassigned as $infos) {
				$result = $user_static->fetch($infos['id']);
				if ($result > 0) {
					$infos = array(
						'id' => $user_static->id,
						'firstname' => $user_static->firstname,
						'lastname' => $user_static->lastname,
						'email' => $user_static->email,
					);

					$result = $this->sendNotification($user, $project, $task, $infos, 4);
					if ($result < 0) {
						$error++;
						break;
					}
				} elseif ($result < 0) {
					$this->error = $user_static->error;
					$this->errors = $user_static->errors;
					$error++;
					break;
				}
			}
		}

		if ($error) {
			$this->db->rollback();
			return -1;
		} else {
			$this->db->commit();
		}

		return 1;
	}

	/**
	 *  Get the event linked to the task
	 *
	 * @param	int				$task_id	Task ID
	 * @return	ActionComm					null if KO otherwise the event object (empty if not found)
	 */
	public function getEvent($task_id)
	{
		dol_syslog(__METHOD__ . " task_id={$task_id}", LOG_DEBUG);
		global $conf, $langs;
		$langs->load('tasktoagenda@tasktoagenda');

		// Clean parameters
		$task_id = $task_id > 0 ? $task_id : 0;

		// Check parameters
		if (empty($task_id)) {
			$langs->load('errors');
			$this->error = $langs->trans('ErrorBadParameters') . " (Task ID)";
			return null;
		}

		$sql = "SELECT ac.id FROM " . MAIN_DB_PREFIX . "actioncomm AS ac WHERE ac.fk_action = '" . self::EVENT_TYPE_ID . "' AND ac.elementtype = '" . self::EVENT_ELEMENT_TYPE . "' AND ac.fk_element = " . $task_id;
		$resql = $this->db->query($sql);
		if (!$resql) {
			$this->errors[] = 'Error ' . $this->db->lasterror();
			dol_syslog(__METHOD__ . " SQL: " . $sql . "; Error: " . $this->db->lasterror(), LOG_ERR);
			return null;
		}

		require_once DOL_DOCUMENT_ROOT . '/comm/action/class/actioncomm.class.php';
		$event = new ActionComm($this->db);

		if ($obj = $this->db->fetch_object($resql)) {
			$result = $event->fetch($obj->id);
			if ($result < 0) {
				$this->error = $event->error;
				$this->errors = $event->errors;
				return null;
			}
		}

		return $event;
	}

	/**
	 *  Send email for notification
	 *
	 * @param	User	$user		User who make the action
	 * @param	Project	$project	Project handler
	 * @param	Task	$task		Task handler
	 * @param	array	$infos		User assigned infos
	 * @param	int		$type		Type notification (0: created, 1: updated, 2: assigned, 3: unassigned, 4: deleted)
	 * @return	int					<0 not ok, >0 ok
	 */
	public function sendNotification(User $user, $project, $task, $infos, $type = 0)
	{
		dol_syslog(__METHOD__ . " user_id={$user->id}, project_id={$project->id}, task_id={$task->id}, infos=" . json_encode($infos) . ", type={$type}", LOG_DEBUG);
		global $conf, $langs;
		$langs->load('tasktoagenda@tasktoagenda', 'projects');

		// Don't notify the user who make the action
		if ($user->id == $infos['id']) return 0;

		$send_to = dolGetFirstLastname($infos['firstname'], $infos['lastname']) . ' <' . $infos['email'] . '>';
		$from = $user->getFullName($langs) . ' <' . $user->email . '>';

		$task_label = $task->ref . ' - ' . $task->label;
		$project_label = $project->ref . ' - ' . $project->title;

		// Subject
		if ($type == 0) {
			$subject = $langs->transnoentitiesnoconv("TaskToAgendaSubjectTaskCreated", $task_label);
		} elseif ($type == 1) {
			$subject = $langs->transnoentitiesnoconv("TaskToAgendaSubjectTaskUpdated", $task_label);
		} elseif ($type == 2) {
			$subject = $langs->transnoentitiesnoconv("TaskToAgendaSubjectTaskAssigned", $task_label);
		} elseif ($type == 3) {
			$subject = $langs->transnoentitiesnoconv("TaskToAgendaSubjectTaskUnassigned", $task_label);
		} else {
			$subject = $langs->transnoentitiesnoconv("TaskToAgendaSubjectTaskDeleted", $task_label);
		}

		// Content
		$message = $langs->transnoentitiesnoconv("Hello") . " " . dolGetFirstLastname($infos['firstname'], $infos['lastname']) . ",";
		$message .= "<br>";

		$message .= "<br>" . $langs->transnoentities("TaskToAgendaBodyNotificationInfos") . ' :';
		$message .= "<br> - " . $langs->transnoentitiesnoconv("DateStart") . " - " . $langs->transnoentitiesnoconv("DateEnd") . " : " . dol_print_date($task->date_start, 'day') . " - " . dol_print_date($task->date_end, 'day');
		$message .= "<br> - " . $langs->transnoentitiesnoconv("Progress") . " : " . ($task->progress >= 0 ? $task->progress . " %" : "");
		$message .= "<br> - " . $langs->transnoentitiesnoconv("Description") . " :<br>" . $task->description;

		$message .= "<br><br>" . $langs->transnoentitiesnoconv("Project") . ' : <a href="' . dol_escape_js(dol_buildpath("/projet/card.php", 2) . "?id={$project->id}") .'">' . $project_label . '</a>';
		$message .= "<br>" . $langs->transnoentitiesnoconv("Task") . ' : <a href="' . dol_escape_js(dol_buildpath("/projet/tasks/task.php", 2) . "?id={$task->id}&withproject={$task->fk_project}") .'">' . $task_label . '</a>';
		$message .= "<br>" . $langs->transnoentitiesnoconv("Author") . " : " . $user->getFullName($langs);

		$error = 0;

		// Send mail
		require_once DOL_DOCUMENT_ROOT . '/core/class/CMailFile.class.php';
		$mailfile = new CMailFile($subject, $send_to, $from, $message);
		if (!empty($mailfile->error)) {
			$out = $mailfile->error;
			$error++;
		} else {
			$result = $mailfile->sendfile();
			if ($result) {
				$out = $langs->trans('MailSuccessfulySent', $mailfile->getValidAddress($from, 2), $mailfile->getValidAddress($send_to, 2));
			} else {
				$langs->load("other");
				$out = '<div class="error">';
				if ($mailfile->error) {
					$out .= $langs->trans('ErrorFailedToSendMail', $from, $send_to);
					$out .= '<br>' . $mailfile->error;
				} else {
					$out .= 'No mail sent. Feature is disabled by option MAIN_DISABLE_ALL_MAILS';
				}
				$out .= '</div>';
				$error++;
			}
		}

		if ($error) {
			$this->errors[] = $out;
		} else {
			$this->successes[] = $out;
		}

		return empty($conf->global->TASKTOAGENDA_NOTIFY_BY_EMAIL_MANDATORY) || !$error ? 1 : -1;
	}

	/**
	 * Method to output saved errors
	 *
	 * @param   string      $separator      Separator between each error
	 * @return	string		                String with errors
	 */
	public function errorsToString($separator = ', ')
	{
		return $this->error . (is_array($this->errors) ? (!empty($this->error) ? $separator : '') . join($separator, $this->errors) : '');
	}
}
